function [freq,psd] = disp2psd(displacement,dt)

% Displacement to power spectral density (PSD)
% usage: [frequency, psd] = disp2psd(displacement data [m], sampling rate [Hz])
% the PSD of time series y(t) is expressed as
% Pk = 2*dt/N *|Yk|^2
% Yk is the Fourier transform of y(t)
% N is the number of samples in each time segment
% The unit of PSD is 1 m^2/s^4/Hz
%
% Reference:
% McNamara D. E., & Buland R. P. (2004). Ambient noise levels in the 
% continental united states. Bulletin of the Seismological Society 
% of America, 94(4): 1517–1527. https://doi.org/10.1785/012003001
%
% Yuechu Wu
% 12131066@mail.sustech.edu.cn
% 2022-09-29

npts = length(displacement);
nextpower = nextpow2(npts);
NFFT = 2^nextpower;

samprate = 1/dt;
freq = samprate/2*linspace(0,1,NFFT/2+1);

velocity= time_differentiate(displacement, samprate);
acceleration= time_differentiate(velocity, samprate);
xdft = fft(acceleration,NFFT);
xdft = xdft(1:NFFT/2+1);
psdx = (1/(samprate*NFFT)).*abs(xdft).^2;
psdx(2:end-1) = 2*psdx(2:end-1);
npts_smooth = floor(NFFT/1000)+1;
psd = smooth(psdx,npts_smooth);

end